/****************************************************************
 * Licensed to the Apache Software Foundation (ASF) under one   *
 * or more contributor license agreements.  See the NOTICE file *
 * distributed with this work for additional information        *
 * regarding copyright ownership.  The ASF licenses this file   *
 * to you under the Apache License, Version 2.0 (the            *
 * "License"); you may not use this file except in compliance   *
 * with the License.  You may obtain a copy of the License at   *
 *                                                              *
 *   http://www.apache.org/licenses/LICENSE-2.0                 *
 *                                                              *
 * Unless required by applicable law or agreed to in writing,   *
 * software distributed under the License is distributed on an  *
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY       *
 * KIND, either express or implied.  See the License for the    *
 * specific language governing permissions and limitations      *
 * under the License.                                           *
 ****************************************************************/
package org.apache.james.smtpserver;

import javax.annotation.Resource;

import org.apache.james.domainlist.api.DomainList;
import org.apache.james.domainlist.api.DomainListException;
import org.apache.james.protocols.smtp.SMTPSession;
import org.apache.james.protocols.smtp.core.AbstractSenderAuthIdentifyVerificationRcptHook;
import org.apache.james.protocols.smtp.hook.HookResult;
import org.apache.james.protocols.smtp.hook.HookReturnCode;
import org.apache.james.smtpserver.netty.SMTPNettySession;
import org.apache.james.user.api.UsersRepository;
import org.apache.james.user.api.UsersRepositoryException;
import org.apache.mailet.MailAddress;

/**
 * Handler which check if the authenticated user is incorrect
 */
public class SenderAuthIdentifyVerificationRcptHook extends AbstractSenderAuthIdentifyVerificationRcptHook {

    private DomainList domains;
    private UsersRepository users;

    @Resource(name = "usersrepository")
    public final void setUsersRepository(UsersRepository users) {
        this.users = users;
    }

    @Resource(name = "domainlist")
    public void setDomainList(DomainList domains) {
        this.domains = domains;
    }

    @Override
    public HookResult doRcpt(SMTPSession session, MailAddress sender, MailAddress rcpt) {
        SMTPNettySession nSession = (SMTPNettySession) session;
        if (nSession.verifyIdentity()) {
            return super.doRcpt(session, sender, rcpt);
        } else {
            return new HookResult(HookReturnCode.DECLINED);
        }
    }

    /**
     * @see org.apache.james.protocols.smtp.core.AbstractSenderAuthIdentifyVerificationRcptHook#isLocalDomain(java.lang.String)
     */
    protected boolean isLocalDomain(String domain) {
        try {
            return domains.containsDomain(domain);
        } catch (DomainListException e) {
            return false;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.james.protocols.smtp.core.
     * AbstractSenderAuthIdentifyVerificationRcptHook#useVirtualHosting()
     */
    protected boolean useVirtualHosting() {
        try {
            return users.supportVirtualHosting();
        } catch (UsersRepositoryException e) {
        }
        return false;
    }
}
